﻿local updateperiod = 0.5 -- display update rate, in seconds
local defaults = {
	profile = {
		bgcolor = {0, 0, 0},
		bgalpha = 0.3,
		bordercolor = {0, 0, 0},
		borderalpha = 0.7,
		showfps = true,
		showlag = true,
		showmem = true,
		showmemrate = true,
		frameopts = {
			x = 0, y = -200,
			anchor = "CENTER",
			scale = 1.0,
		}
	}
}

local OptionHouse = LibStub("OptionHouse-1.1")
local LegoBlock = DongleStub("LegoBlock-Beta0")
PerfBlock = DongleStub("Dongle-1.1"):New("PerfBlock")
local PerfBlock = PerfBlock
local LB

function PerfBlock:CreateUI()
	local function colorset(key, val)
		self.db.profile[key].r = val.r
		self.db.profile[key].g = val.g
		self.db.profile[key].b = val.b
		self:ApplySettings()
	end
	local config = {
		{
			group = "Appearance",
			type = "color",
			text = "Background Color",
			var = "bgcolor",
			set = colorset,
		},
		{
			group = "Appearance",
			type = "color",
			text = "Border Color",
			var = "bordercolor",
			set = colorset,
		},
		{
			group = "Appearance",
			type = "slider",
			format = "Background opacity: %.2f",
			var = "bgalpha",
			step = 0.05,
			minText = "0%",
			maxText = "100%",
			min = 0.0,
			max = 1.0,
		},
		{
			group = "Appearance",
			type = "slider",
			format = "Border opacity: %.2f",
			var = "borderalpha",
			step = 0.05,
			minText = "0%",
			maxText = "100%",
			min = 0.0,
			max = 1.0,
		},
		{
			group = "Appearance",
			type = "slider",
			format = "Scale: %.2f",
			var = "scale",
			step = 0.1,
			minText = "10%",
			maxText = "300%",
			min = 0.1,
			max = 3.0,
			set = function(key, val)
				self.db.profile.frameopts[key] = val
				LB:RestorePosition()
			end,
			get = function(key)
				return self.db.profile.frameopts[key]
			end,
		},
		{
			group = "Show",
			type = "check",
			text = "Frames Per Second",
			var = "showfps",
		},
		{
			group = "Show",
			type = "check",
			text = "Network Latency",
			var = "showlag",
		},
		{
			group = "Show",
			type = "check",
			text = "Total Used AddOn Memory",
			var = "showmem",
		},
		{
			group = "Show",
			type = "check",
			text = "AddOn Memory Rate (In KiB per second)",
			var = "showmemrate",
		},
	}

	local HouseAuthority = LibStub("HousingAuthority-1.2")

	return HouseAuthority:CreateConfiguration(config, {
		set = function(key, val)
			self.db.profile[key] = val
			self:ApplySettings()
		end,
		get = function(key)
			return self.db.profile[key]
		end,
		columns = 2,
	})
end

function PerfBlock:Initialize()
	self.db = self:InitializeDB("PerfBlockDB", defaults, "Global")

	-- Convert from old format
	if type(self.db.profile.bgcolor) == "string" then
		local r, g, b, a = self.db.profile.bgcolor:match("(%d.%d) (%d.%d) (%d.%d) (%d.%d)")
		self.db.profile.bgcolor = {r = tonumber(r), g = tonumber(g), b = tonumber(b)}
		self.db.profile.bgalpha = tonumber(a)
	end
	if type(self.db.profile.bordercolor) == "string" then
		local r, g, b, a = self.db.profile.bordercolor:match("(%d.%d) (%d.%d) (%d.%d) (%d.%d)")
		self.db.profile.bordercolor = {r = tonumber(r), g = tonumber(g), b = tonumber(b)}
		self.db.profile.borderalpha = tonumber(a)
	end

	LB = LegoBlock:New("PerfBlock", "----", nil, self.db.profile.frameopts)

	local slash = self:InitializeSlashCommand("PerfBlock", "PERFBLOCK", "perfblock", "pb")
	slash:RegisterSlashHandler("config - Open config dialog.", "config$", "OpenConfig")
	slash:RegisterSlashHandler("resetpos - Reset Position.", "resetpos$", "ResetPosition")
	slash:RegisterSlashHandler("profile - Show Current Profile.", "profile$", "ShowProfile")
	slash:RegisterSlashHandler("profile setchar - Set this character's profile to '" .. self.db.keys.char .. "'.", "profile setchar$", "SetChar")
	slash:InjectDBCommands(self.db, "copy", "delete", "list", "reset", "set")

	local title, author, version = GetAddOnMetadata("PerfBlock", "Title"),
	                               GetAddOnMetadata("PerfBlock", "Author"),
	                               GetAddOnMetadata("PerfBlock", "Version")
	local OHObj = OptionHouse:RegisterAddOn("PerfBlock", title, author, version)
	OHObj:RegisterCategory("General", self, "CreateUI")
end

function PerfBlock:ShowProfile()
	self:PrintF("Current profile: '%s'", self.db:GetCurrentProfile())
end

function PerfBlock:SetChar()
	self.db:SetProfile(self.db.keys.char)
end

function PerfBlock:OpenConfig()
	OptionHouse:Open("PerfBlock", "General")
end

function PerfBlock:ResetPosition()
	self.db.profile.frameopts.x = defaults.profile.frameopts.x
	self.db.profile.frameopts.y = defaults.profile.frameopts.y
	self.db.profile.frameopts.anchor = defaults.profile.frameopts.anchor
	LB:RestorePosition()
end

function PerfBlock:ApplySettings()
	local db = self.db.profile
	LB:SetBackdropColor(db.bgcolor.r, db.bgcolor.g, db.bgcolor.b, db.bgalpha)
	LB:SetBackdropBorderColor(db.bordercolor.r, db.bordercolor.g, db.bordercolor.b, db.borderalpha)
	LB:RestorePosition()
	self.Update()
end

function PerfBlock:ProfileMessage(msg, db, dongle)
	if db == self.db then
		LB:SetDB(db.profile.frameopts)
		self:ApplySettings()
	end
end

function PerfBlock:ModifierChange(event, modifier, state)
	if (modifier == "ALT") and (UnitAffectingCombat("player") ~= 1) then
		if LB.isMoving then
			LB:GetScript("OnDragStop")(LB)
		end
		LB:EnableMouse(state == 1)
	end
end

function PerfBlock:Enable()
	LB:EnableMouse(false)
	self:RegisterEvent("MODIFIER_STATE_CHANGED", "ModifierChange")
	self:RegisterMessage("DONGLE_PROFILE_RESET", "ProfileMessage")
	self:RegisterMessage("DONGLE_PROFILE_CHANGED", "ProfileMessage")
	self:RegisterMessage("DONGLE_PROFILE_COPIED", "ProfileMessage")

	LB:SetScript("OnMouseDown", function(frame, button)
		if button == "RightButton" then
			PerfBlock:OpenConfig()
		end
	end)

	self:ScheduleRepeatingTimer("PerfBlock", self.Update, updateperiod)
	self:ApplySettings()
end


local string_format = string.format
local math_modf = math.modf
local select = select

local RGBPercToHex = function(r, g, b)
	return string_format("%02x%02x%02x", r*255, g*255, b*255)
end

local ColorGradient = function(perc, ...)
	if perc >= 1 then
		local r, g, b = select(select("#", ...) - 2, ...)
		return r, g, b
	elseif perc <= 0 then
		local r, g, b = ...
		return r, g, b
	end

	local num = select("#", ...) / 3

	local segment, relperc = math_modf(perc*(num-1))
	local r1, g1, b1, r2, g2, b2 = select((segment*3)+1, ...)

	return r1 + (r2-r1)*relperc, g1 + (g2-g1)*relperc, b1 + (b2-b1)*relperc
end

local GetFramerate = GetFramerate
local GetNetStats = GetNetStats
local collectgarbage = collectgarbage
local prevmem = collectgarbage("count")
local legotbl = {}
function PerfBlock.Update()
	for i=1,#legotbl do
		legotbl[i] = nil
	end

	local db = PerfBlock.db.profile
	if db.showfps then
		local fps = GetFramerate()
		local r, g, b = ColorGradient(fps/75, 1,0,0, 1,1,0, 0,1,0)
		table.insert(legotbl, ("|cff%s%.1f|r FPS"):format(RGBPercToHex(r, g, b), fps))
	end
	if db.showlag then
		local lag = select(3, GetNetStats())
		local r, g, b = ColorGradient(lag/1000, 0,1,0, 1,1,0, 1,0,0)
		table.insert(legotbl, ("|cff%s%d|r ms"):format(RGBPercToHex(r, g, b), lag))
	end

	local mem = collectgarbage("count")
	local deltamem = mem - prevmem
	prevmem = mem
	if db.showmem then
		local r, g, b = ColorGradient(mem/(60*1024), 0,1,0, 1,1,0, 1,0,0)
		table.insert(legotbl, ("|cff%s%.2f|r MiB"):format(RGBPercToHex(r, g, b), mem/1024))
	end
	if db.showmemrate then
		local r, g, b = ColorGradient(deltamem/15, 0,1,0, 1,1,0, 1,0,0)
		table.insert(legotbl, ("|cff%s%.2f|r KiB/sec"):format(RGBPercToHex(r, g, b), deltamem))
	end

	LB:SetText(table.concat(legotbl, "\n"))
	LB.Text:SetWidth(LB:GetWidth())
	LB:SetHeight(LB.Text:GetHeight() + 15)
end

-- vim: set sw=4 ts=4 noet sts=0:
